`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    09:41:03 04/18/2016 
// Design Name: 
// Module Name:    Main 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module Main(
	input clk_100M, input RxD, output TxD, output [7:0]Leds, input BTNS, input BTNL, input BTNR, 
	output [3:0] SegmentDrivers,
	output [7:0] SevenSegment,
	output JC1, output JD1,
	input [7:0]Switch,
	input activate
	);

wire RxD_data_ready;
wire [7:0]RxD_data;
reg [7:0]TxD_data;
wire RxD_idle;
wire RxD_endofpacket;
reg TxD_start = 0;
wire TxD_busy;

reg [7:0]a1 = 8'd255;
reg [7:0]d = 8'd10;
reg [31:0]numElements = 32'd100000;

reg [1:0]savingState;

async_receiver receiver(clk_100M, RxD, RxD_data_ready, RxD_data, RxD_idle, RxD_endofpacket);
async_transmitter transmitter(clk_100M, TxD_start, TxD_data, TxD, TxD_busy);

wire PB_Center_Down, PB_Left_Down, PB_Right_Down; 
Debouncer DebounceC(clk_100M, BTNS, , PB_Center_Down,);
Debouncer DebounceL(clk_100M, BTNL, , PB_Left_Down,);
Debouncer DebounceR(clk_100M, BTNR, , PB_Right_Down,);

parameter M = 20; // how many calc modules will be instantiated (max number of elements in sequence)

wire [(M*64)-1:0]calcResults; // One output line for each calc module
wire [(M*64)-1:0]validLines; // One output line for each calc module
wire [(M*64)-1:0]memReadData; // Data coming from memory.
wire [9:0]memReadAddr; // Memory address to read.

assign memReadAddr = CurrentReadAddr;

wire dataValid;
wire [31:0]groupOffset;
wire [4:0]memAddr;
ControlUnit control(.done(done), .groupOffset(groupOffset), .validOut(dataValid), .memAddrOut(memAddr), .n(numElements), .activate(activate), .M(M), .clock(clk_100M));

//assign activate = PB_Center_Down;
//assign JC1 = activate;
assign Leds[0] = done;
assign JD1 = done;

// Define a number of calc modules. Each one will process one element in the sequence:
generate
genvar n;
for(n = 1; n <= M; n = n+1) begin: CalcInstance
	ArithmeticUnit arith(.clock(clk_100M), .validIn(dataValid), .memAddrIn(memAddr), .validOut(validLines[n*64-1-:64]), .a1(a1), .d(d), .N(n), .elementOut(calcResults[(n*64-1)-:64]), .groupOffset(groupOffset));
	mem_1k mem(
		.clka(clk_100M),
		.wea(~done),
		.addra(memAddr),
		.dina(calcResults[(n*64-1)-:64]),
		.clkb(clk_100M),
		.addrb(memReadAddr),
		.doutb(memReadData[n*64-1-:64])
	);
end
endgenerate

wire [63:0]outputVal;


reg transmissionComplete = 0;
assign Leds[7] = transmissionComplete;
reg [7:0]CurrentMemBlock = 1; // which memory block are we currently transmitting from?
reg [9:0]CurrentReadAddr = 0; // which address in the memory block are we transmitting from?
reg [3:0]byteToTransmit = 0; // which byte (0 to 7) of the 64-bit integer are we transmitting?

assign outputVal = memReadData[CurrentMemBlock*64-1-:64];

SS_Driver SSDriver(
	clk_100M, activate,
	
	outputVal[15:0],
	
	SegmentDrivers, 
	SevenSegment
);

always@(posedge clk_100M) begin

if(done && ~transmissionComplete) begin
	// Calculation is done. Start sending values over UART:
	if(~TxD_busy) begin
		if(CurrentReadAddr == 1023) begin
			transmissionComplete <= 1;
			CurrentReadAddr <= 0;
			byteToTransmit <= 0;
		end
		else begin
			// CurrentReadAddr <= 1024
			if(CurrentMemBlock > M) begin
				CurrentMemBlock <= 1;
				CurrentReadAddr <= CurrentReadAddr + 1;
			end
			else begin
				// CurrentMemBlock <= M
				if(byteToTransmit >= 8) begin
					byteToTransmit <= 0;
					CurrentMemBlock <= CurrentMemBlock + 1;
				end
				else begin
					// byte < 8
					TxD_data = outputVal[(byteToTransmit*8)+7-:8];
					byteToTransmit <= byteToTransmit + 1;
					TxD_start <= 1;
				end
			end
		end
	end // TxD_busy
	else begin
		TxD_start <= 0;
	end
end




/*
*  if the left / right button is pressed,
*  increment / decrement the memory address whose value is shown on the 7 segment display:
*/
if(PB_Right_Down) begin
	// Right button pressed. Increment address and update display:
	if(CurrentMemBlock >= M) begin
		CurrentMemBlock <= 1;
		CurrentReadAddr <= CurrentReadAddr + 1'b1;
	end
	else begin
		CurrentMemBlock <= CurrentMemBlock + 1'b1;
	end
end else if (PB_Left_Down) begin
	if(CurrentMemBlock <= 0) begin
		CurrentMemBlock <= M;
		CurrentReadAddr <= CurrentReadAddr - 1'b1;
	end
	else begin
		CurrentMemBlock <= CurrentMemBlock - 1'b1;
	end
end

	
	
end


endmodule
